﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Windows.Input;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace WpfApplication3
{

    public class WPFMap : WpfMap
    {
        bool initialized = false;
        WSIInMemoryFeatureLayer layer;
        LayerOverlay lov;

        public WPFMap()
        {
            this.Loaded += new RoutedEventHandler(WPFMapLoaded);
        }

        void WPFMapLoaded(object sender, RoutedEventArgs e)
        {

            if (!initialized && !System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
            {
                initialized = true;
                this.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;
                MapUnit = GeographyUnit.DecimalDegree;
                lov = new LayerOverlay();
                lov.TransitionEffect = TransitionEffect.None;

                layer = new WSIInMemoryFeatureLayer();
                layer.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                layer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
                layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                lov.Layers.Add(layer);
                Overlays.Add(lov);

                MyEditOverlay me = new MyEditOverlay();
                me.MapFeatureDragged += new EventHandler<FeatureDraggedEditInteractiveOverlayEventArgs>(EditOverlay_FeatureDragged);
                me.MapFeatureDragging += new EventHandler<FeatureDraggingEditInteractiveOverlayEventArgs>(EditOverlay_FeatureDragging);
                EditOverlay = me;
                EditOverlay.EditShapesLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
                EditOverlay.EditShapesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                EditOverlay.DragControlPointsLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital3;
                EditOverlay.DragControlPointsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                EditOverlay.CanAddVertex = false;
                EditOverlay.CanRemoveVertex = false;
                EditOverlay.CanReshape = false;
                EditOverlay.CanResize = false;
                EditOverlay.CanRotate = false;
                EditOverlay.CanDrag = false;   

                layer.Open();
                RectangleShape r = ExtentHelper.GetBoundingBoxOfItems(layer.QueryTools.GetAllFeatures(ReturningColumnsType.NoColumns));
                SetZoomLevelScales(r);
                this.CurrentExtent = r;
                CurrentScale = MaximumScale;
                layer.Close();
            }
        }
       
        void EditOverlay_FeatureDragging(object sender, FeatureDraggingEditInteractiveOverlayEventArgs e)
        {
            Console.WriteLine("FeatureDragging={0},{1},{2}", e.DraggingFeature.GetShape(), e.SourceControlPoint, e.TargetControlPoint);

            layer.Open();
            double boundary = this.CurrentResolution * 4;
            RectangleShape r = 
                new RectangleShape(e.TargetControlPoint.X - boundary, e.TargetControlPoint.Y + boundary, e.TargetControlPoint.X + boundary, e.TargetControlPoint.Y - boundary);
            Collection<Feature> features = layer.QueryTools.GetFeaturesIntersecting(r, ReturningColumnsType.NoColumns);
            Console.WriteLine("Found={0}", features.Count);
            layer.Close();

            if (features.Count == 1 && !r.Intersects(e.DraggingFeature) )
            {
               
                    //Removes the "selected point"
                    EditOverlay.EditShapesLayer.Open();
                    EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                    EditOverlay.EditShapesLayer.BuildIndex();
                    EditOverlay.EditShapesLayer.Close();

                    EditOverlay.CanDrag = false;
                    EditOverlay.CalculateAllControlPoints();

                    ContextMenu.IsOpen = true;
       
            }
            else
            {
                ContextMenu.IsOpen = false;
            }

            Refresh(EditOverlay);
        }

        void EditOverlay_FeatureDragged(object sender, FeatureDraggedEditInteractiveOverlayEventArgs e)
        {
            Console.WriteLine("FeatureDragged");

            //Removes the "selected point"
            EditOverlay.EditShapesLayer.Open();
            EditOverlay.EditShapesLayer.InternalFeatures.Clear();
            EditOverlay.EditShapesLayer.BuildIndex();
            EditOverlay.EditShapesLayer.Close();
            EditOverlay.CanDrag = false;
            EditOverlay.CalculateAllControlPoints();

            Refresh(EditOverlay);
        }

   
        protected override void OnMapClick(MapClickWpfMapEventArgs e)
        {
            base.OnMapClick(e);
            layer.Open();
            
            // Select a point
            double boundary = this.CurrentResolution * 4;
            RectangleShape r = new RectangleShape(e.WorldX - boundary, e.WorldY + boundary, e.WorldX + boundary, e.WorldY - boundary);
            Collection<Feature> features = layer.QueryTools.GetFeaturesIntersecting(r, ReturningColumnsType.NoColumns);
            if (features.Count == 1 && e.MouseButton == MapMouseButton.Left)
            {
                // Add selected point
                EditOverlay.EditShapesLayer.Open();
                EditOverlay.EditShapesLayer.EditTools.BeginTransaction();
                EditOverlay.EditShapesLayer.EditTools.Add(features[0]);
                EditOverlay.EditShapesLayer.EditTools.CommitTransaction();
                EditOverlay.EditShapesLayer.Close();
                EditOverlay.CanDrag = true;
                EditOverlay.CalculateAllControlPoints();
                Refresh(EditOverlay);
            }
            else if (features.Count == 0)
            {
                //Removes the "selected point"
                EditOverlay.EditShapesLayer.Open();
                EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                EditOverlay.EditShapesLayer.BuildIndex();
                EditOverlay.EditShapesLayer.Close();

                EditOverlay.CanDrag = false;
                EditOverlay.CalculateAllControlPoints();
                Refresh(EditOverlay);
            }

            layer.Close();
        }

        #region SetZoomLevelScales
        private void SetZoomLevelScales(RectangleShape rs)
        {
            double start = ExtentHelper.GetScale(rs, (float)ActualWidth, GeographyUnit.DecimalDegree);
            ZoomLevelScales.Clear();
            MaximumScale = start;
      
            int count = 0;
            while (count++ < 20)
            {
                ZoomLevelScales.Add(start);
                start = start / 2;
                if (start < 1.0)
                    break;
            }
            if (ZoomLevelScales.Count < 20)
                for (int i = ZoomLevelScales.Count; i < 20; i++)
                {
                    ZoomLevelScales.Add(1.0);
                }

            MinimumScale = ZoomLevelScales[ZoomLevelScales.Count - 1];
        }
        #endregion
    }

    #region MyEditOverlay
    public class MyEditOverlay : EditInteractiveOverlay
     {
         private bool featureDragging = false;
         public event EventHandler<FeatureDraggedEditInteractiveOverlayEventArgs> MapFeatureDragged;
         public event EventHandler<FeatureDraggingEditInteractiveOverlayEventArgs> MapFeatureDragging;


         protected override void EndEditingCore(PointShape targetPointShape)
         {
             Console.WriteLine("EndEditingCore({0}) enter/exit", targetPointShape);
             base.EndEditingCore(targetPointShape);
             if (featureDragging)
             {
                 featureDragging = false;
                 if (MapFeatureDragged != null)
                     MapFeatureDragged(this, new FeatureDraggedEditInteractiveOverlayEventArgs(targetPointShape.GetFeature()));

             }
         }
         protected override void OnFeatureDragging(FeatureDraggingEditInteractiveOverlayEventArgs e)
         {
             //Console.WriteLine("OnFeatureDragging() enter/exit");
             base.OnFeatureDragging(e);
             featureDragging = true;
             if (MapFeatureDragging != null)
                MapFeatureDragging(this, e);
         }

     }
    #endregion

    #region FeatureSource
    public class WSIInMemoryFeatureSource : FeatureSource
     {
         public Collection<Feature> allFeatures;
         double incr = .000005;
         public WSIInMemoryFeatureSource()
         {
             allFeatures = new Collection<Feature>();

             double y = 35.553;
             for (int i = 0; i < 10; i++)
             {
                 double x = -99;

                 for (int j = 0; j < 100; j++)
                 {
                     Feature f = new Feature(x, y);
                     allFeatures.Add(f);
                     x += incr;
                 }
                 y += incr;
             }
         }
         protected override Collection<Feature> GetAllFeaturesCore(IEnumerable<string> returningColumnNames)
         {
             return allFeatures;
         }
         protected override int GetCountCore()
         {
             return allFeatures.Count;
         }
     }

     public class WSIInMemoryFeatureLayer : FeatureLayer
     {
         public WSIInMemoryFeatureLayer()
         {
             FeatureSource = new WSIInMemoryFeatureSource();
             SetupTools();
         }
     }
     #endregion
 }
